@@ -6,6 +6,20 @@
  *	Support for enhanced MLS infrastructure.
  *
  * Copyright (C) 2004-2005 Trusted Computer Solutions, Inc.
+ *
+ *  This library is free software; you can redistribute it and/or
+ *  modify it under the terms of the GNU Lesser General Public
+ *  License as published by the Free Software Foundation; either
+ *  version 2.1 of the License, or (at your option) any later version.
+ *
+ *  This library is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ *  Lesser General Public License for more details.
+ *
+ *  You should have received a copy of the GNU Lesser General Public
+ *  License along with this library; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 /* FLASK */
@@ -41,7 +55,7 @@ int mls_convert_context(policydb_t * oldp,
 int mls_compute_sid(policydb_t *policydb,
 		    context_struct_t *scontext,
 		    context_struct_t *tcontext,
-		    security_class_t tclass,
+		    sepol_security_class_t tclass,
 		    uint32_t specified,
 		    context_struct_t *newcontext);
 
